@@ -13,6 +13,8 @@
 #include <config.h>
 #endif
 
+#include <string.h>
+
 #include <panel-applet.h>
 #include <panel-applet-gconf.h>
 
@@ -45,14 +47,15 @@ typedef struct {
 	GtkWidget *display_workspaces_toggle;
 	GtkWidget *all_workspaces_radio;
 	GtkWidget *current_only_radio;
-	GtkWidget *num_rows_spin;
+	GtkWidget *num_rows_spin;	       /* for vertical layout this is cols */
+	GtkWidget *label_row_col;
 	GtkWidget *num_workspaces_spin;
 	GtkWidget *workspaces_tree;
 
 	GtkListStore *workspaces_store;
 	
 	GtkOrientation orientation;
-	int n_rows;
+	int n_rows;				/* for vertical layout this is cols */
 	WnckPagerDisplayMode display_mode;
 	gboolean display_all;
 	int size;
@@ -121,6 +124,8 @@ applet_change_orient (PanelApplet       *applet,
   
 	pager->orientation = new_orient;
 	pager_update (pager);
+	if (pager->label_row_col) 
+		gtk_label_set_text(GTK_LABEL(pager->label_row_col), pager->orientation == GTK_ORIENTATION_HORIZONTAL ? _("Rows") : _("Columns"));	
 }
 
 
@@ -152,9 +157,9 @@ destroy_pager(GtkWidget * widget, PagerData *pager)
 }
 
 static const BonoboUIVerb pager_menu_verbs [] = {
-	BONOBO_UI_UNSAFE_VERB ("PagerProperties", display_properties_dialog),
-	BONOBO_UI_UNSAFE_VERB ("PagerHelp",       display_help_dialog),
-	BONOBO_UI_UNSAFE_VERB ("PagerAbout",      display_about_dialog),
+	BONOBO_UI_UNSAFE_VERB ("PagerPreferences", display_properties_dialog),
+	BONOBO_UI_UNSAFE_VERB ("PagerHelp",        display_help_dialog),
+	BONOBO_UI_UNSAFE_VERB ("PagerAbout",       display_about_dialog),
         BONOBO_UI_VERB_END
 };
 
@@ -282,7 +287,7 @@ fill_pager_applet(PanelApplet *applet)
 	GError *error;
 	gboolean display_names;
 	
-	panel_applet_add_preferences (applet, "/schemas/apps/pager-applet/prefs", NULL);
+	panel_applet_add_preferences (applet, "/schemas/apps/pager_applet/prefs", NULL);
 	
 	pager = g_new0 (PagerData, 1);
 
@@ -338,8 +343,7 @@ fill_pager_applet(PanelApplet *applet)
 		break;
 	}
 
-	/* TODO: Needs to get the screen number from DISPLAY or the panel. */
-	pager->screen = wnck_screen_get (0);
+	pager->screen = wnck_screen_get_default ();
 
 	/* because the pager doesn't respond to signals at the moment */
 	wnck_screen_force_update (pager->screen);
@@ -391,7 +395,7 @@ display_help_dialog (BonoboUIComponent *uic,
 {
 	GError *error = NULL;
 
-	gnome_help_display_desktop (NULL, "pager", "pager", NULL, &error);
+	gnome_help_display_desktop (NULL, "workspace-switcher", "workspace-switcher", NULL, &error);
 	if (error) {
 		g_warning ("help error: %s\n", error->message);
 		g_error_free (error);
@@ -404,7 +408,7 @@ display_about_dialog (BonoboUIComponent *uic,
 		      const gchar       *verbname)
 {
 	static GtkWidget *about = NULL;
-	GdkPixbuf *pixbuf;
+	GdkPixbuf *pixbuf = NULL;
 	gchar *file;
 	
 	static const gchar *authors[] =
@@ -412,6 +416,10 @@ display_about_dialog (BonoboUIComponent *uic,
 		"Alexander Larsson <alla@lysator.liu.se>",
 		NULL
 	};
+	const char *documenters [] = {
+	  NULL
+	};
+	const char *translator_credits = _("translator_credits");
 
 	if (about != NULL) {
 		gtk_widget_show (about);
@@ -419,21 +427,16 @@ display_about_dialog (BonoboUIComponent *uic,
 		return;
 	}
 
-	pixbuf = NULL;
-
-	/* TODO: This should not use gnome-clock! */
-	file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, "gnome-clock.png", TRUE, NULL);
-	if (!file) {
-		g_warning (G_STRLOC ": gnome-clock.png cannot be found");
-		pixbuf = gdk_pixbuf_new_from_file (file, NULL);
-	}
+	file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, "gnome-workspace.png", TRUE, NULL);
+	pixbuf = gdk_pixbuf_new_from_file (file, NULL);
+	g_free(file);
 
 	about = gnome_about_new (_("Workspace Switcher"), "1.0",
 				 _("(c) 2001 Red Hat, Inc."),
-				 _("The Workspace Switcher shows you a small version of your workspaces that lets you manage the windows."),
+				 _("The Workspace Switcher shows you a small version of your workspaces that lets you manage your windows."),
 				 authors,
-				 NULL, /* documenters */
-				 NULL, /* translator_credits */
+				 documenters,
+				 strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
 				 pixbuf);
 	
 	gtk_window_set_wmclass (GTK_WINDOW (about), "pager", "Pager");
@@ -493,7 +496,7 @@ update_workspaces_model (PagerData *pager)
 
 	gtk_list_store_clear (pager->workspaces_store);
 	for (i = 0; i < nr_ws; i++) {
-		workspace = wnck_workspace_get (i);
+		workspace = wnck_screen_get_workspace (pager->screen, i);
 		gtk_list_store_append (pager->workspaces_store, &iter);
 		gtk_list_store_set (pager->workspaces_store,
 				    &iter,
@@ -607,6 +610,7 @@ setup_dialog (GladeXML  *xml,
 	pager->all_workspaces_radio = WID ("all_workspaces_radio");
 	pager->current_only_radio = WID ("current_only_radio");
 	pager->num_rows_spin = WID ("num_rows_spin");
+	pager->label_row_col = WID("label_row_col");
 	pager->num_workspaces_spin = WID ("num_workspaces_spin");
 	pager->workspaces_tree = WID ("workspaces_tree_view");
 
@@ -640,6 +644,7 @@ setup_dialog (GladeXML  *xml,
 			  (GCallback) num_rows_value_changed, pager);
 
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (pager->num_rows_spin), pager->n_rows);
+	gtk_label_set_text(GTK_LABEL(pager->label_row_col), pager->orientation == GTK_ORIENTATION_HORIZONTAL ? _("Rows") : _("Columns"));
 
 	g_signal_connect (pager->properties_dialog, "delete_event",
 			  G_CALLBACK (delete_event),
@@ -672,7 +677,7 @@ setup_dialog (GladeXML  *xml,
 	
 	nr_ws = wnck_screen_get_workspace_count (pager->screen);
 	for (i = 0; i < nr_ws; i++) {
-		g_signal_connect (G_OBJECT (wnck_workspace_get (i)), "name_changed",
+		g_signal_connect (G_OBJECT (wnck_screen_get_workspace (pager->screen, i)), "name_changed",
 				  (GCallback) workspace_renamed, pager);
 	}
 }
@@ -688,6 +693,9 @@ display_properties_dialog (BonoboUIComponent *uic,
 		xml = glade_xml_new (PAGER_GLADEDIR "/pager.glade", NULL, NULL);
 		pager->properties_dialog = glade_xml_get_widget (xml, "pager_properties_dialog");
 
+		g_object_add_weak_pointer (G_OBJECT (pager->properties_dialog), 
+					   (gpointer *) &pager->properties_dialog);
+
 		setup_dialog (xml, pager);
 		
 		g_object_unref (G_OBJECT (xml));
