@@ -13,6 +13,8 @@
 #include <config.h>
 #endif
 
+#include <string.h>
+
 #include <panel-applet.h>
 #include <panel-applet-gconf.h>
 
@@ -28,7 +30,6 @@
 
 typedef struct {
 	GtkWidget *applet;
-
 	GtkWidget *tasklist;
 	
 	WnckScreen *screen;
@@ -39,6 +40,7 @@ typedef struct {
   
 	GtkOrientation orientation;
 	int size;
+	gint maximum_size;
 
 	/* Properties: */
 	GtkWidget *properties_dialog;
@@ -50,8 +52,11 @@ typedef struct {
 	GtkWidget *move_minimized_radio;
 	GtkWidget *change_workspace_radio;
 
+	GtkWidget *minimum_size_spin;
+	GtkWidget *maximum_size_spin;
+
 	/* gconf listeners id */
-	guint listeners [3];
+	guint listeners [5];
 } TasklistData;
 
 static void display_properties_dialog (BonoboUIComponent *uic,
@@ -64,7 +69,6 @@ static void display_about_dialog      (BonoboUIComponent *uic,
 				       TasklistData      *tasklist,
 				       const gchar       *verbname);
 
-
 static void
 tasklist_update (TasklistData *tasklist)
 {
@@ -91,6 +95,8 @@ applet_change_orient (PanelApplet       *applet,
 		      TasklistData      *tasklist)
 {
 	GtkOrientation new_orient;
+    	WnckTasklist *wncktl = WNCK_TASKLIST (tasklist->tasklist);
+	gint minimum_size = 0;
   
 	switch (orient)	{
 	case PANEL_APPLET_ORIENT_LEFT:
@@ -108,6 +114,21 @@ applet_change_orient (PanelApplet       *applet,
 		return;
   
 	tasklist->orientation = new_orient;
+	/* we need to unset minimum size on the wnck tasklist */
+	if (tasklist->orientation == GTK_ORIENTATION_HORIZONTAL)
+	{
+	    	/* unset the minimum height and reset minimum width */
+	        minimum_size = wnck_tasklist_get_minimum_height (wncktl); 
+	        wnck_tasklist_set_minimum_height (wncktl, -1);
+	        wnck_tasklist_set_minimum_width (wncktl, minimum_size);
+	}
+	else
+	{
+	    	/* unset the minimum width and reset minimum height */
+	        minimum_size = wnck_tasklist_get_minimum_width (wncktl); 
+	        wnck_tasklist_set_minimum_width (wncktl, -1);
+	        wnck_tasklist_set_minimum_height (wncktl, minimum_size);
+	}
 
 	tasklist_update (tasklist);
 }
@@ -155,31 +176,23 @@ destroy_tasklist(GtkWidget * widget, TasklistData *tasklist)
 	gconf_client_notify_remove (client, tasklist->listeners[0]);
 	gconf_client_notify_remove (client, tasklist->listeners[1]);
 	gconf_client_notify_remove (client, tasklist->listeners[2]);
+	gconf_client_notify_remove (client, tasklist->listeners[3]);
+	gconf_client_notify_remove (client, tasklist->listeners[4]);
 	
 	tasklist->listeners[0] = 0;
 	tasklist->listeners[1] = 0;
 	tasklist->listeners[2] = 0;
+	tasklist->listeners[3] = 0;
+	tasklist->listeners[4] = 0;
 }
 
 static const BonoboUIVerb tasklist_menu_verbs [] = {
-	BONOBO_UI_UNSAFE_VERB ("TasklistProperties", display_properties_dialog),
-	BONOBO_UI_UNSAFE_VERB ("TasklistHelp",       display_help_dialog),
-	BONOBO_UI_UNSAFE_VERB ("TasklistAbout",      display_about_dialog),
+	BONOBO_UI_UNSAFE_VERB ("TasklistPreferences", display_properties_dialog),
+	BONOBO_UI_UNSAFE_VERB ("TasklistHelp",        display_help_dialog),
+	BONOBO_UI_UNSAFE_VERB ("TasklistAbout",       display_about_dialog),
         BONOBO_UI_VERB_END
 };
 
-static const char tasklist_menu_xml [] =
-	"<popup name=\"button3\">\n"
-	"   <menuitem name=\"Tasklist Properties Item\" verb=\"TasklistProperties\" _label=\"Properties ...\"\n"
-	"             pixtype=\"stock\" pixname=\"gtk-properties\"/>\n"
-	"   <menuitem name=\"Tasklist Help Item\" verb=\"TasklistHelp\" _label=\"Help\"\n"
-	"             pixtype=\"stock\" pixname=\"gtk-help\"/>\n"
-	"   <menuitem name=\"Tasklist About Item\" verb=\"TasklistAbout\" _label=\"About ...\"\n"
-	"             pixtype=\"stock\" pixname=\"gnome-stock-about\"/>\n"
-	"</popup>\n";
-
-
-
 static void
 tasklist_properties_update_content_radio (TasklistData *tasklist)
 {
@@ -329,6 +342,45 @@ move_unminimized_windows_changed (GConfClient  *client,
 	tasklist_update_unminimization_radio (tasklist);
 }
 
+/* GConf callback for changes in minimum_size */
+static void
+minimum_size_changed (GConfClient *client, guint cnxn_id,
+		      GConfEntry *entry, TasklistData *tasklist)
+{
+    	WnckTasklist *wncktl = WNCK_TASKLIST (tasklist->tasklist);
+	gint value;
+	GtkSpinButton *button = GTK_SPIN_BUTTON (tasklist->minimum_size_spin);
+
+	if (!entry->value || entry->value->type != GCONF_VALUE_INT)
+		return;
+	
+	value = gconf_value_get_int (entry->value);
+	
+	gtk_spin_button_set_value (button, value);
+	if (tasklist->orientation == GTK_ORIENTATION_HORIZONTAL)
+		wnck_tasklist_set_minimum_width (wncktl, value);
+	else
+		wnck_tasklist_set_minimum_height (wncktl, value);
+}
+
+/* GConf callback for changes in maximum_size */
+static void
+maximum_size_changed (GConfClient  *client, guint cnxn_id,
+                      GConfEntry   *entry, TasklistData *tasklist)
+{
+	gint value;
+	GtkSpinButton *button = GTK_SPIN_BUTTON (tasklist->maximum_size_spin);
+
+	if (!entry->value || entry->value->type != GCONF_VALUE_INT)
+		return;
+	
+	value = gconf_value_get_int (entry->value);
+
+	gtk_spin_button_set_value (button, value);
+	tasklist->maximum_size = value;
+        gtk_widget_queue_resize (GTK_WIDGET (tasklist->applet));
+}
+    
 static void
 setup_gconf (TasklistData *tasklist)
 {
@@ -360,8 +412,71 @@ setup_gconf (TasklistData *tasklist)
 				tasklist,
 				NULL, NULL);
 	g_free (key);
-}
+	
+	key = panel_applet_gconf_get_full_key (PANEL_APPLET (tasklist->applet),
+					       "minimum_size");
+	tasklist->listeners[3] = gconf_client_notify_add(client, key,
+				(GConfClientNotifyFunc)minimum_size_changed,
+				tasklist,
+				NULL, NULL);
+	g_free (key);
+	key = panel_applet_gconf_get_full_key (PANEL_APPLET (tasklist->applet),
+					       "maximum_size");
+	tasklist->listeners[4] = gconf_client_notify_add(client, key,
+				(GConfClientNotifyFunc)maximum_size_changed,
+				tasklist,
+				NULL, NULL);
+	g_free (key);}
 
+static void
+applet_size_request (GtkWidget      *widget,
+		     GtkRequisition *requisition,
+		     TasklistData   *tasklist)
+{
+	int len;
+	const int *size_hints;
+	GtkRequisition child_req;
+	int i;
+	int maximum_size;
+	int *new_size_hints;
+	int new_len = 0;
+    	WnckTasklist *wncktl = WNCK_TASKLIST (tasklist->tasklist);
+	gint minimum_size = 0;
+	
+	
+	if (tasklist->orientation == GTK_ORIENTATION_HORIZONTAL)
+		minimum_size = wnck_tasklist_get_minimum_width (wncktl);
+	else
+		minimum_size = wnck_tasklist_get_minimum_height (wncktl);
+	
+	gtk_widget_get_child_requisition (tasklist->applet, &child_req);
+	
+	size_hints = wnck_tasklist_get_size_hint_list (wncktl, &len);
+	new_size_hints = g_new0 (int, len);
+	
+	/* size_hints is an array of (max, min) int pairs
+	 * where min(i) > max (i+1)
+	 * convert it to clipped values
+	 */
+	maximum_size = tasklist->maximum_size - minimum_size;
+	g_assert (maximum_size >= 0);
+
+	for (i = 0; i < len; i += 2) {
+		if (size_hints[i + 1] <= maximum_size) {
+		        /* this one should be stored */
+			if (size_hints[i] > maximum_size)
+			    	new_size_hints[new_len] = maximum_size;
+			else
+				new_size_hints[new_len] = size_hints[i];
+			new_size_hints[new_len + 1] = size_hints[i + 1];
+			new_len += 2;
+		}
+	}
+        panel_applet_set_size_hints (PANEL_APPLET (tasklist->applet),
+		                     new_size_hints, 
+				     new_len, child_req.width - 1);
+	g_free (new_size_hints);
+}
 
 gboolean
 fill_tasklist_applet(PanelApplet *applet)
@@ -370,7 +485,9 @@ fill_tasklist_applet(PanelApplet *applet)
 	GError *error;
 	GConfValue *value;
 
-	panel_applet_add_preferences (applet, "/schemas/apps/tasklist-applet/prefs", NULL);
+	gint sizepref;
+
+	panel_applet_add_preferences (applet, "/schemas/apps/tasklist_applet/prefs", NULL);
 	
 	tasklist = g_new0 (TasklistData, 1);
 
@@ -403,7 +520,7 @@ fill_tasklist_applet(PanelApplet *applet)
 		g_error_free (error);
 		tasklist->move_unminimized_windows = TRUE; /* Default value */
 	}
-	
+
 	tasklist->size = panel_applet_get_size (applet);
 	switch (panel_applet_get_orient (applet)) {
 	case PANEL_APPLET_ORIENT_LEFT:
@@ -417,25 +534,50 @@ fill_tasklist_applet(PanelApplet *applet)
 		break;
 	}
 
-	/* FIXME: Needs to get the screen number from DISPLAY or the panel. */
-	tasklist->screen = wnck_screen_get (0);
+	tasklist->screen = wnck_screen_get_default ();
 
 	/* because the tasklist doesn't respond to signals at the moment */
 	wnck_screen_force_update (tasklist->screen);
 
 	tasklist->tasklist = wnck_tasklist_new (tasklist->screen);
 
+	/* get size preferences */
+	error = NULL;
+	sizepref = panel_applet_gconf_get_int (applet, "minimum_size", 
+					       &error);
+        if (error == NULL) {
+		if (tasklist->orientation == GTK_ORIENTATION_HORIZONTAL)
+			wnck_tasklist_set_minimum_width (WNCK_TASKLIST (tasklist->tasklist), sizepref);	  
+		else
+			wnck_tasklist_set_minimum_height (WNCK_TASKLIST (tasklist->tasklist), sizepref);	  
+	}
+	else
+		g_error_free (error);
+
+	error = NULL;
+	sizepref = panel_applet_gconf_get_int (applet, "maximum_size", 
+					       &error);
+        if (error == NULL) 
+	        tasklist->maximum_size = sizepref;
+	else
+		g_error_free (error);
+
 	g_signal_connect (G_OBJECT (tasklist->tasklist), "destroy",
 			  G_CALLBACK (destroy_tasklist),
 			  tasklist);
 
+	g_signal_connect (G_OBJECT (tasklist->applet), "size_request",
+			  G_CALLBACK (applet_size_request),
+			  tasklist);
 	tasklist_update (tasklist);
 	gtk_widget_show (tasklist->tasklist);
 
 	gtk_container_add (GTK_CONTAINER (tasklist->applet), tasklist->tasklist);
 	
 	panel_applet_set_flags (PANEL_APPLET (tasklist->applet),
-				PANEL_APPLET_EXPAND_MAJOR | PANEL_APPLET_EXPAND_MINOR);
+				PANEL_APPLET_EXPAND_MAJOR |
+				PANEL_APPLET_EXPAND_MINOR |
+				PANEL_APPLET_HAS_HANDLE);
 
 	gtk_widget_show (tasklist->applet);
 
@@ -453,7 +595,12 @@ fill_tasklist_applet(PanelApplet *applet)
 			  G_CALLBACK (applet_change_background),
 			  tasklist);
 	
-	panel_applet_setup_menu (PANEL_APPLET (tasklist->applet), tasklist_menu_xml, tasklist_menu_verbs, tasklist);
+	panel_applet_setup_menu_from_file (PANEL_APPLET (tasklist->applet),
+					   NULL,
+					   "GNOME_TasklistApplet.xml",
+					   NULL,
+					   tasklist_menu_verbs, 
+					   tasklist);
 	
 	return TRUE;
 }
@@ -467,7 +614,7 @@ display_help_dialog (BonoboUIComponent *uic,
 	GError *error = NULL;
 
 	gnome_help_display_desktop (
-			NULL, "tasklist", "tasklist", NULL, &error);
+			NULL, "window-list", "window-list", NULL, &error);
 	if (error) {
 		g_warning ("help error: %s\n", error->message);
 		g_error_free (error);
@@ -480,7 +627,7 @@ display_about_dialog (BonoboUIComponent *uic,
 		      const gchar       *verbname)
 {
 	static GtkWidget *about = NULL;
-	GdkPixbuf *pixbuf;
+	GdkPixbuf *pixbuf = NULL;
 	gchar *file;
 	
 	static const gchar *authors[] =
@@ -488,6 +635,10 @@ display_about_dialog (BonoboUIComponent *uic,
 		"Alexander Larsson <alla@lysator.liu.se>",
 		NULL
 	};
+	const char *documenters [] = {
+	  NULL
+	};
+	const char *translator_credits = _("translator_credits");
 
 	if (about != NULL) {
 		gtk_widget_show (about);
@@ -495,21 +646,16 @@ display_about_dialog (BonoboUIComponent *uic,
 		return;
 	}
 
-	pixbuf = NULL;
-
-	/* FIXME: This should not use gnome-clock! */
-	file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, "gnome-clock.png", TRUE, NULL);
-	if (!file) {
-		g_warning (G_STRLOC ": gnome-clock.png cannot be found");
-		pixbuf = gdk_pixbuf_new_from_file (file, NULL);
-	}
+	file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, "gnome-windows.png", TRUE, NULL);
+	pixbuf = gdk_pixbuf_new_from_file (file, NULL);
+	g_free(file);
 
 	about = gnome_about_new (_("Window List"), "1.0",
 				 _("(c) 2001 Red Hat, Inc"),
-				 _("The Window List shows a list of all visible windows and let you browse them."),
+				 _("The Window List shows a list of all visible windows and lets you browse them."),
 				 authors,
-				 NULL, /* documenters */
-				 NULL, /* translator_credits */
+				 documenters,
+				 strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
 				 pixbuf);
 	
 	gtk_window_set_wmclass (GTK_WINDOW (about), "tasklist", "Tasklist");
@@ -556,6 +702,43 @@ display_all_workspaces_toggled (GtkToggleButton *button,
 				     NULL);
 }
 
+/* called when minimum width spin button is changed
+ * check if it exceeds max width
+ * saves numeric GConf preference values
+ */
+static void
+spin_minimum_size_changed (GtkSpinButton *button, TasklistData *tasklist)
+{
+	GtkSpinButton *max_b = GTK_SPIN_BUTTON (tasklist->maximum_size_spin);
+	PanelApplet *applet = PANEL_APPLET (tasklist->applet);
+	gint prop_value = gtk_spin_button_get_value (button);
+	gint max_size = gtk_spin_button_get_value (max_b);
+	
+	/* check if we exceed max width */
+	if (prop_value > max_size)
+	        panel_applet_gconf_set_int (applet, "maximum_size", 
+			                    prop_value, NULL);
+	panel_applet_gconf_set_int (applet, "minimum_size", prop_value, NULL);
+}
+
+/* called when maximum width spin button is changed
+ * check if we drop below min width
+ * saves numeric GConf preference values
+ */
+static void
+spin_maximum_size_changed (GtkSpinButton *button, TasklistData *tasklist)
+{
+	GtkSpinButton *min_b = GTK_SPIN_BUTTON (tasklist->minimum_size_spin);
+	PanelApplet *applet = PANEL_APPLET (tasklist->applet);
+	gint prop_value = gtk_spin_button_get_value (button);
+	gint min_size = gtk_spin_button_get_value (min_b);
+
+	/* check if we drop below min width */
+	if (prop_value < min_size)
+		panel_applet_gconf_set_int (applet, "minimum_size", 
+					    prop_value, NULL);
+	panel_applet_gconf_set_int (applet, "maximum_size", prop_value, NULL);
+}
 
 #define WID(s) glade_xml_get_widget (xml, s)
 
@@ -565,6 +748,8 @@ setup_dialog (GladeXML     *xml,
 {
 	GConfClient *client;
 	GtkWidget *button;
+	gint sizepref;
+	GError *error;
 	
 	client = gconf_client_get_default ();
 
@@ -576,6 +761,9 @@ setup_dialog (GladeXML     *xml,
 	tasklist->move_minimized_radio = WID ("move_minimized_radio");
 	tasklist->change_workspace_radio = WID ("change_workspace_radio");
 
+	tasklist->minimum_size_spin = WID ("minimum_size");
+	tasklist->maximum_size_spin = WID ("maximum_size");
+
 	/* Window grouping: */
 	button = get_grouping_button (tasklist, tasklist->grouping);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
@@ -585,6 +773,15 @@ setup_dialog (GladeXML     *xml,
 			   "group_value", "auto");
 	g_object_set_data (G_OBJECT (tasklist->always_group_radio),
 			   "group_value", "always");
+	error = NULL;
+	/* FIXME: what does one do in case of errors here ? */
+	sizepref = panel_applet_gconf_get_int (PANEL_APPLET (tasklist->applet), 
+	    				       "minimum_size", NULL);
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (tasklist->minimum_size_spin), sizepref);
+	sizepref = panel_applet_gconf_get_int (PANEL_APPLET (tasklist->applet), 
+	    				       "maximum_size", NULL);
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (tasklist->maximum_size_spin), sizepref);
+
 	g_signal_connect (G_OBJECT (tasklist->never_group_radio), "toggled",
 			  (GCallback) group_windows_toggled, tasklist);
 	g_signal_connect (G_OBJECT (tasklist->auto_group_radio), "toggled",
@@ -602,11 +799,22 @@ setup_dialog (GladeXML     *xml,
 	g_signal_connect (G_OBJECT (tasklist->show_all_radio), "toggled",
 			  (GCallback) display_all_workspaces_toggled, tasklist);
 
+	g_signal_connect (G_OBJECT (tasklist->minimum_size_spin), 
+		          "value_changed", 
+			  (GCallback) spin_minimum_size_changed,
+			  tasklist);
+	g_signal_connect (G_OBJECT (tasklist->maximum_size_spin), 
+		          "value_changed", 
+			  (GCallback) spin_maximum_size_changed,
+			  tasklist);
 	g_signal_connect_swapped (WID ("done_button"), "clicked",
-				  (GCallback) gtk_widget_hide, tasklist->properties_dialog);
+				  (GCallback) gtk_widget_hide, 
+				  tasklist->properties_dialog);
 
 }
 
+
+
 static void 
 display_properties_dialog (BonoboUIComponent *uic,
 			   TasklistData      *tasklist,
@@ -618,6 +826,9 @@ display_properties_dialog (BonoboUIComponent *uic,
 		xml = glade_xml_new (TASKLIST_GLADEDIR "/tasklist.glade", NULL, NULL);
 		tasklist->properties_dialog = glade_xml_get_widget (xml, "tasklist_properties_dialog");
 
+		g_object_add_weak_pointer (G_OBJECT (tasklist->properties_dialog),
+					   (void**) &tasklist->properties_dialog);
+
 		setup_dialog (xml, tasklist);
 		
 		g_object_unref (G_OBJECT (xml));
@@ -625,3 +836,5 @@ display_properties_dialog (BonoboUIComponent *uic,
 
 	gtk_window_present (GTK_WINDOW (tasklist->properties_dialog));
 }
+
+
