@@ -25,8 +25,6 @@
 #include "panel-gconf.h"
 #include "panel-config-global.h"
 #include "session.h"
-#include "status.h"
-#include "swallow.h"
 #include "panel-applet-frame.h"
 
 #define SMALL_ICON_SIZE 20
@@ -47,14 +45,15 @@ static GConfEnumStringPair object_type_enum_map [] = {
 	{ APPLET_DRAWER,   "drawer-object" },
 	{ APPLET_MENU,     "menu-object" },
 	{ APPLET_LOGOUT,   "logout-object" },
-	{ APPLET_SWALLOW,  "swallow-object" },
 	{ APPLET_LAUNCHER, "launcher-object" }, 
 	{ APPLET_EMPTY,    "empty-object" },
 	{ APPLET_LOCK,     "lock-object" },
-	{ APPLET_STATUS,   "status-object" },
 	{ APPLET_BONOBO,   "bonobo-applet" },
 };
 
+static GSList *queued_position_saves = NULL;
+static guint   queued_position_source = 0;
+
 static void
 move_applet_callback (GtkWidget *widget, AppletInfo *info)
 {
@@ -72,11 +71,12 @@ move_applet_callback (GtkWidget *widget, AppletInfo *info)
 }
 
 static void
-panel_applet_clean_gconf (AppletInfo *info)
+panel_applet_clean_gconf (AppletInfo *info,
+			  gboolean    clean_gconf)
 {
 	GConfClient *client;
 	GSList      *id_list, *l;
-	char        *temp_key = NULL;
+	const char  *temp_key = NULL;
 	const char  *profile;
 
 	g_return_if_fail (info != NULL);
@@ -86,41 +86,38 @@ panel_applet_clean_gconf (AppletInfo *info)
 	profile = panel_gconf_get_profile ();
 
 	if (info->type == APPLET_BONOBO)
-		temp_key = panel_gconf_general_key (profile, "applet-id-list");
+		temp_key = panel_gconf_general_key (profile, "applet_id_list");
 	else
-		temp_key = panel_gconf_general_key (profile, "object-id-list");
+		temp_key = panel_gconf_general_key (profile, "object_id_list");
 
         id_list = gconf_client_get_list (
 			panel_gconf_get_client (), temp_key,
 			GCONF_VALUE_STRING, NULL);
 
         for (l = id_list; l; l = l->next)
-                if (!strcmp (info->gconf_key, (char *) l->data))
+                if (strcmp (info->gconf_key, (char *) l->data) == 0)
                         break;
 
         if (l) {
-		char *tmp;
-
-                id_list = g_slist_remove_link (id_list, l);
-
 		g_free (l->data);
-		g_slist_free (l);
+		l->data = NULL;
+                id_list = g_slist_delete_link (id_list, l);
 
                 gconf_client_set_list (client, temp_key, GCONF_VALUE_STRING, id_list, NULL);
 
 		if (info->type == APPLET_BONOBO)
-			tmp = g_strdup_printf ("/apps/panel/profiles/%s/applets/%s",
-					       profile, info->gconf_key);
+			temp_key = panel_gconf_sprintf (
+					"/apps/panel/profiles/%s/applets/%s",
+					profile, info->gconf_key);
 		else
-			tmp = g_strdup_printf ("/apps/panel/profiles/%s/objects/%s",
-					       profile, info->gconf_key);
+			temp_key = panel_gconf_sprintf (
+					"/apps/panel/profiles/%s/objects/%s",
+					profile, info->gconf_key);
 
-		panel_gconf_clean_dir (client, tmp);
-
-		g_free (tmp);
+		if (clean_gconf)
+			panel_gconf_clean_dir (client, temp_key);
         }
 
-        g_free (temp_key);
 	panel_g_slist_deep_free (id_list);
 
 	g_free (info->gconf_key);
@@ -129,7 +126,8 @@ panel_applet_clean_gconf (AppletInfo *info)
 
 /*destroy widgets and call the above cleanup function*/
 void
-panel_applet_clean (AppletInfo *info)
+panel_applet_clean (AppletInfo *info,
+		    gboolean    clean_gconf)
 {
 	g_return_if_fail (info != NULL);
 
@@ -143,16 +141,16 @@ panel_applet_clean (AppletInfo *info)
 	if (info->widget) {
 		GtkWidget *widget = info->widget;
 
-		if (info->type == APPLET_STATUS)
-			status_applet_put_offscreen (info->data);
-
 		info->widget = NULL;
 		gtk_widget_destroy (widget);
 	}
 
 	info->data = NULL;
 
-	panel_applet_clean_gconf (info);
+	panel_applet_clean_gconf (info, clean_gconf);
+
+	queued_position_saves =
+		g_slist_remove (queued_position_saves, info);
 
 	g_free (info);
 }
@@ -164,15 +162,11 @@ applet_idle_remove (gpointer data)
 
 	info->remove_idle = 0;
 
-	panel_applet_save_position (info, info->gconf_key);
+	if (info->type == APPLET_BONOBO)
+		panel_applet_frame_set_clean_remove (
+				PANEL_APPLET_FRAME (info->data), TRUE);
 
-	if (info->type == APPLET_SWALLOW) {
-		Swallow *swallow = info->data;
-
-		swallow->clean_remove = TRUE;
-	}
-
-	panel_applet_clean (info);
+	panel_applet_clean (info, TRUE);
 
 	return FALSE;
 }
@@ -223,17 +217,6 @@ applet_callback_callback(GtkWidget *widget, gpointer data)
 			panel_show_help ("drawers", NULL);
 		}
 		break;
-	case APPLET_SWALLOW:
- 		if (strcmp (menu->name, "help") == 0)
-			panel_show_help ("specialobjects", "SWALLOWEDAPP");
-#if 0
-		if(strcmp(menu->name,"properties")==0) {
-			Swallow *swallow = info->data;
-			g_assert(swallow);
-			swallow_properties(swallow); /* doesn't exist yet*/
-		}
-#endif
-		break; 
 	case APPLET_MENU:
 		if(strcmp(menu->name,"properties")==0)
 			menu_properties(menu->info->data);
@@ -253,11 +236,11 @@ applet_callback_callback(GtkWidget *widget, gpointer data)
 		break;
 	case APPLET_LOCK: {
                 /*
-		  <jwz> Blank Screen Now
-		  <jwz> Lock Screen Now
-		  <jwz> Kill Daemon
-		  <jwz> Restart Daemon
-		  <jwz> Preferences
+		  <jwz> Activate Screensaver
+		  <jwz> Lock Screen
+		  <jwz> Kill Screensaver Daemon
+		  <jwz> Restart Screensaver Daemon
+		  <jwz> Properties
 		  <jwz> (or "configuration" instead?  whatever word you use)
 		  <jwz> those should do xscreensaver-command -activate, -lock, -exit...
 		  <jwz> and "xscreensaver-command -exit ; xscreensaver &"
@@ -267,9 +250,9 @@ applet_callback_callback(GtkWidget *widget, gpointer data)
 		gboolean freeit = FALSE;
 		if (strcmp (menu->name, "help") == 0)
 			panel_show_help ("specialobjects", "LOCKBUTTON");
-		else if (!strcmp (menu->name, "restart")) {
+		else if (strcmp (menu->name, "restart") == 0) {
 			command = "xscreensaver-command -exit ; xscreensaver &";
-		} else if (!strcmp (menu->name, "prefs")) {
+		} else if (strcmp (menu->name, "prefs") == 0) {
 			command = "xscreensaver-demo";
 		} else {
 			command = g_strdup_printf ("xscreensaver-command -%s",
@@ -286,10 +269,6 @@ applet_callback_callback(GtkWidget *widget, gpointer data)
 		if (strcmp (menu->name, "help") == 0)
 			panel_show_help ("specialobjects", "LOGOUTBUTTON");
 		break;
-	case APPLET_STATUS:
-		if (strcmp (menu->name, "help") == 0)
-			panel_show_help ("specialobjects", "STATUSDOC");
-		break;
 	case APPLET_BONOBO:
 		/*
 		 * Applet's menu's are handled differently
@@ -320,7 +299,7 @@ panel_applet_get_callback (GList      *user_menu,
 	for (l = user_menu; l; l = l->next) {
 		AppletUserMenu *menu = l->data;
 
-		if (!strcmp (menu->name, name))
+		if (strcmp (menu->name, name) == 0)
 			return menu;
 	}
 
@@ -520,120 +499,68 @@ add_to_submenus (AppletInfo *info,
 }
 
 static GtkWidget *
-applet_setup_panel_menu (gboolean is_basep)
-{
-	GtkWidget *menuitem;
-	GtkWidget *panel_menu;
-	GdkPixbuf *pixbuf;
-
-	menuitem = gtk_image_menu_item_new ();
-
-	pixbuf = panel_make_menu_icon ("gnome-panel.png",
-				       NULL /* fallback */,
-				       SMALL_ICON_SIZE /* size */,
-				       NULL /* long_operation */);
-
-	if (pixbuf != NULL) {
-		GtkWidget *image = NULL;
-		
-		image = gtk_image_new_from_pixbuf (pixbuf);
-		g_object_unref (pixbuf);
-		
-		setup_menuitem (menuitem, 
-				image,
-				_("Panel"));
-	} else {
-		g_message (_("Cannot find pixmap file %s"), "gnome-panel.png");
-
-		setup_menuitem (menuitem, NULL, _("Panel"));
-	}
-
-	panel_menu = panel_menu_new ();
-
-	make_panel_submenu (panel_menu, TRUE, is_basep);
-
-	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), panel_menu);
-
-	return menuitem;
-}
-
-void
-panel_applet_create_menu (AppletInfo *info,
-			  gboolean    is_basep)
+panel_applet_create_menu (AppletInfo *info)
 {
+	GtkWidget *menu;
 	GtkWidget *menuitem;
 	GList     *l;
 
-	info->menu = g_object_ref (panel_menu_new ());
-	gtk_object_sink (GTK_OBJECT (info->menu));
+	menu = g_object_ref (panel_menu_new ());
+	gtk_object_sink (GTK_OBJECT (menu));
+
+	/* connect the deactivate signal, so that we can "re-allow" 
+	 * autohide when the menu is deactivated.
+	 */
+	g_signal_connect (menu, "deactivate",
+			  G_CALLBACK (applet_menu_deactivate), info);
+
+	if (info->user_menu) {
+		for (l = info->user_menu; l; l = l->next) {
+			AppletUserMenu *user_menu = (AppletUserMenu *)l->data;
+
+			add_to_submenus (info, "", user_menu->name, user_menu, 
+					 menu, info->user_menu);
+		}
+	}
 
 	if (!commie_mode) {
 		GtkWidget *image;
 
+		menuitem = gtk_separator_menu_item_new ();
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+		gtk_widget_show (menuitem);
+
 		menuitem = gtk_image_menu_item_new ();
 
 		image = gtk_image_new_from_stock (GTK_STOCK_REMOVE,
 						  GTK_ICON_SIZE_MENU);
 
-		setup_menuitem (menuitem, image , _("Remove from panel"));
+		setup_menuitem (menuitem, image , _("Remove From Panel"));
 
-		g_signal_connect (G_OBJECT (menuitem),
-				  "activate",
-				  G_CALLBACK (applet_remove_callback),
-				  info);
+		g_signal_connect (menuitem, "activate",
+				  G_CALLBACK (applet_remove_callback), info);
 
-		gtk_menu_shell_append (GTK_MENU_SHELL (info->menu), menuitem);
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
 
-		menuitem = gtk_image_menu_item_new();
+		menuitem = gtk_image_menu_item_new ();
 
-		/*
-		 * FIXME: should have a "Move" pixmap.
+		/* FIXME: should have a "Move" pixmap.
 		 */
 		setup_menuitem (menuitem, NULL, _("Move"));
 
-		g_signal_connect (G_OBJECT (menuitem),
-				  "activate",
-				  G_CALLBACK (move_applet_callback),
-				  info);
+		g_signal_connect (menuitem, "activate",
+				  G_CALLBACK (move_applet_callback), info);
 
-		gtk_menu_shell_append (GTK_MENU_SHELL (info->menu), menuitem);
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
 	}
 
-	menuitem = applet_setup_panel_menu (is_basep);
-
-	gtk_menu_shell_append (GTK_MENU_SHELL (info->menu), menuitem);
-
-	if (info->user_menu) {
-		menuitem = gtk_image_menu_item_new ();
-
-		gtk_menu_shell_append (GTK_MENU_SHELL (info->menu), menuitem);
-
-		gtk_widget_set_sensitive (menuitem, FALSE);
-
-		gtk_widget_show (menuitem);
-	}
-
-	for (l = info->user_menu; l; l = l->next) {
-		AppletUserMenu *menu = (AppletUserMenu *)l->data;
-
-		add_to_submenus (info, "", menu->name, menu, 
-				 info->menu, info->user_menu);
-	}
-
-	/*
-	 * connect the deactivate signal, so that we can "re-allow" 
-	 * autohide when the menu is deactivated.
-	 */
-	g_signal_connect (G_OBJECT (info->menu),
-			    "deactivate",
-			     G_CALLBACK (applet_menu_deactivate),
-			     info);
+	return menu;
 }
 
-static void
-applet_menu_set_data_recursively (GtkMenu     *menu,
-				  const gchar *key,
-				  gpointer     data)
+void
+panel_applet_menu_set_recurse (GtkMenu     *menu,
+			       const gchar *key,
+			       gpointer     data)
 {
 	GList *children;
 	GList *l;
@@ -646,7 +573,8 @@ applet_menu_set_data_recursively (GtkMenu     *menu,
 		GtkWidget *submenu = GTK_MENU_ITEM (l->data)->submenu;
 
 		if (submenu)
-			applet_menu_set_data_recursively (GTK_MENU (submenu), key, data);
+			panel_applet_menu_set_recurse (
+				GTK_MENU (submenu), key, data);
 	}
 
 	g_list_free (children);
@@ -663,7 +591,7 @@ applet_show_menu (AppletInfo     *info,
 	panel = get_panel_parent (info->widget);
 
 	if (!info->menu)
-		panel_applet_create_menu (info, BASEP_IS_WIDGET (panel));
+		info->menu = panel_applet_create_menu (info);
 
 	g_assert (info->menu);
 
@@ -674,9 +602,9 @@ applet_show_menu (AppletInfo     *info,
 
 	info->menu_age = 0;
 
-	applet_menu_set_data_recursively (GTK_MENU (info->menu),
-					  "menu_panel",
-					  info->widget->parent);
+	panel_applet_menu_set_recurse (GTK_MENU (info->menu),
+				       "menu_panel",
+				       info->widget->parent);
 
 	if (!GTK_WIDGET_REALIZED (info->menu))
 		gtk_widget_show (info->menu);
@@ -698,22 +626,10 @@ applet_do_popup_menu (GtkWidget      *widget,
 	if (panel_applet_in_drag)
 		return FALSE;
 
-	switch (info->type) {
-	case APPLET_BONOBO:
-			break;
-	case APPLET_SWALLOW: {
-		GtkHandleBox *handle_box;
+	if (info->type == APPLET_BONOBO)
+		return FALSE;
 
-		handle_box = GTK_HANDLE_BOX (((Swallow *)info->data)->handle_box);
-
-		if (handle_box->child_detached)
-			applet_show_menu (info, event);
-		}
-		break;
-	default:
-		applet_show_menu (info, event);
-		break;
-	}
+	applet_show_menu (info, event);
 
 	return TRUE;
 }
@@ -735,14 +651,10 @@ applet_button_press (GtkWidget      *widget,
 		     GdkEventButton *event,
 		     AppletInfo     *info)
 {
-	if (event->button == 3) {
+	if (event->button == 3)
 		return applet_do_popup_menu (widget, event, info);
-	}
 
-	if (BUTTON_IS_WIDGET (widget))
-		return FALSE;
-	else
-		return TRUE;
+	return FALSE;
 }
 
 static void
@@ -758,12 +670,12 @@ applet_destroy (GtkWidget *w, AppletInfo *info)
 
 	if (info->type == APPLET_DRAWER) {
 		Drawer *drawer = info->data;
-		g_assert(drawer);
-		if(drawer->drawer) {
-			GtkWidget *dw = drawer->drawer;
+
+		if (drawer->drawer) {
+			PANEL_WIDGET (BASEP_WIDGET (drawer->drawer)->panel)->master_widget = NULL;
+
+			gtk_widget_destroy (drawer->drawer);
 			drawer->drawer = NULL;
-			PANEL_WIDGET(BASEP_WIDGET(dw)->panel)->master_widget = NULL;
-			gtk_widget_destroy(dw);
 		}
 	} else if (info->type == APPLET_LAUNCHER) {
 		Launcher    *launcher = info->data;
@@ -807,59 +719,85 @@ applet_destroy (GtkWidget *w, AppletInfo *info)
 	/* If this was not called from the panel_applet_clean
 	 * itself.  That is if the widget entry was still set */
 	if (old_widget != NULL)
-		panel_applet_clean (info);
+		panel_applet_clean (info, TRUE);
 }
 
-static char *
+static G_CONST_RETURN char *
 panel_applet_get_full_gconf_key (AppletType   type,
 				 const gchar *profile,
 				 const gchar *object_id,
 				 const gchar *key)
 {
 	PanelGConfKeyType  gconf_type;
-	char              *retval = NULL;
 
 	if (type == APPLET_BONOBO)
 		gconf_type = PANEL_GCONF_APPLETS;
 	else
 		gconf_type = PANEL_GCONF_OBJECTS;
 
-	retval = panel_gconf_full_key (gconf_type, profile, object_id, key);
-
-	return retval;
+	return panel_gconf_full_key (gconf_type, profile, object_id, key);
 }
 
 typedef struct {
 	PanelWidget *panel_widget;
 	int          position;
 	char        *unique_id;
+	gulong       destroy_handler;
 } PanelAppletToLoad;
 
 static GSList *panel_applets_to_load = NULL;
 
+static void
+whack_applet_to_load (PanelAppletToLoad *applet)
+{
+	if (applet->destroy_handler > 0)
+		g_signal_handler_disconnect (applet->panel_widget,
+					     applet->destroy_handler);
+	applet->destroy_handler = 0;
+
+	g_free (applet->unique_id);
+	applet->unique_id = NULL;
+
+	g_free (applet);
+}
+
 static gboolean
 panel_applet_load_idle_handler (gpointer dummmy)
 {
 	PanelAppletToLoad *applet;
 
-	if (!panel_applets_to_load)
+	if (panel_applets_to_load == NULL)
 		return FALSE;
 
 	applet = (PanelAppletToLoad *) panel_applets_to_load->data;
+	panel_applets_to_load->data = NULL;
 	panel_applets_to_load =
-		g_slist_remove (panel_applets_to_load, applet);
+		g_slist_delete_link (panel_applets_to_load,
+				     panel_applets_to_load);
 
 	panel_applet_frame_load_from_gconf (
 			applet->panel_widget,
 			applet->position,
 			applet->unique_id);
 
-	g_free (applet->unique_id);
-	g_free (applet);
+	whack_applet_to_load (applet);
 
 	return TRUE;
 }
 
+static void
+panel_destroyed_while_loading (GtkWidget *panel, gpointer data)
+{
+	PanelAppletToLoad *applet = data;
+
+	applet->destroy_handler = 0;
+
+	panel_applets_to_load =
+		g_slist_remove (panel_applets_to_load, applet);
+
+	whack_applet_to_load (applet);
+}
+
 static void
 panel_applet_load_from_unique_id (AppletType   type,
 				  GConfClient *gconf_client,
@@ -868,16 +806,15 @@ panel_applet_load_from_unique_id (AppletType   type,
 {
 	PanelWidget *panel_widget;
 	AppletType   applet_type;
+	const char  *temp_key;
 	char        *type_string;
-	char        *temp_key;
 	char        *panel_id;
 	int          position;
 	gboolean     right_stick;
 
 	temp_key = panel_applet_get_full_gconf_key (
-			type, profile, unique_id, "object-type");
+			type, profile, unique_id, "object_type");
 	type_string = gconf_client_get_string (gconf_client, temp_key, NULL);
-	g_free (temp_key);
 
 	if (!gconf_string_to_enum (object_type_enum_map,
 				   type_string,
@@ -892,25 +829,25 @@ panel_applet_load_from_unique_id (AppletType   type,
 	temp_key = panel_applet_get_full_gconf_key (
 			type, profile, unique_id, "position");
 	position = gconf_client_get_int (gconf_client, temp_key, NULL);
-	g_free (temp_key);
 	
 	temp_key = panel_applet_get_full_gconf_key (
-			type, profile, unique_id, "panel-id");
+			type, profile, unique_id, "panel_id");
 	panel_id = gconf_client_get_string (gconf_client, temp_key, NULL);
-	g_free (temp_key);
 
 	temp_key = panel_applet_get_full_gconf_key (
-			type, profile, unique_id, "panel-right-stick");
+			type, profile, unique_id, "panel_right_stick");
 	right_stick = gconf_client_get_bool (gconf_client, temp_key, NULL);
-	g_free (temp_key);
 
 	panel_widget = panel_widget_get_by_id (panel_id);
 	g_free (panel_id);
 
-	/*
-	 * A hack from the old panel to make sure the applet is on the right
-	 */
-	position += right_stick ? G_MAXINT/2 : 0;
+	if (!panel_widget) {
+		g_warning ("panel_applet_load: can't find the panel for this object\n");
+		return;
+	}
+
+	if (right_stick)
+		position = panel_widget->size - position;
 
 	switch (applet_type) {
 	case APPLET_BONOBO: {
@@ -918,9 +855,13 @@ panel_applet_load_from_unique_id (AppletType   type,
 
 		applet = g_new0 (PanelAppletToLoad, 1);
 
-		applet->panel_widget = panel_widget;
-		applet->position     = position;
-		applet->unique_id    = g_strdup (unique_id);
+		applet->panel_widget    = panel_widget;
+		applet->position        = position;
+		applet->unique_id       = g_strdup (unique_id);
+		applet->destroy_handler =
+				g_signal_connect (panel_widget, "destroy",
+						  G_CALLBACK (panel_destroyed_while_loading),
+						  applet);
 
 		if (!panel_applets_to_load)
 			g_idle_add (panel_applet_load_idle_handler, NULL);
@@ -931,9 +872,6 @@ panel_applet_load_from_unique_id (AppletType   type,
 	case APPLET_DRAWER:
 		drawer_load_from_gconf (panel_widget, position, unique_id);
 		break;
-	case APPLET_SWALLOW:
-		swallow_load_from_gconf (panel_widget, position, unique_id);
-		break;
 	case APPLET_MENU:
 		menu_load_from_gconf (panel_widget, position, unique_id);
 		break;
@@ -946,9 +884,6 @@ panel_applet_load_from_unique_id (AppletType   type,
 	case APPLET_LOCK:
 		load_lock_applet (panel_widget, position, TRUE, unique_id);
 		break;
-	case APPLET_STATUS:
-		load_status_applet (panel_widget, position, TRUE, unique_id);
-		break;
 	default:
 		break;
 	}
@@ -960,15 +895,15 @@ panel_applet_load_list (AppletType type)
 	GConfClient *client;
 	GSList      *id_list, *l;
 	const char  *profile;
-	char        *temp_key = NULL;
+	const char  *temp_key = NULL;
 	
 	client  = panel_gconf_get_client ();
 	profile = panel_gconf_get_profile ();
 
 	if (type == APPLET_BONOBO)
-		temp_key = panel_gconf_general_key (profile, "applet-id-list");
+		temp_key = panel_gconf_general_key (profile, "applet_id_list");
 	else
-        	temp_key = panel_gconf_general_key (profile, "object-id-list");
+        	temp_key = panel_gconf_general_key (profile, "object_id_list");
 
         id_list = gconf_client_get_list (
 			panel_gconf_get_client (), temp_key,
@@ -977,7 +912,6 @@ panel_applet_load_list (AppletType type)
         for (l = id_list; l; l = l->next)
 		panel_applet_load_from_unique_id (type, client, profile, (char *) l->data);
 
-        g_free (temp_key);
 	panel_g_slist_deep_free (id_list);
 }
 
@@ -988,31 +922,72 @@ panel_applet_load_applets_from_gconf (void)
 	panel_applet_load_list (APPLET_EMPTY);
 }
 
+static gboolean
+panel_applet_position_save_timeout (gpointer dummy)
+{
+	GSList *l;
+
+	queued_position_source = 0;
+
+	for (l = queued_position_saves; l; l = l->next) {
+		AppletInfo *info = l->data;
+
+		panel_applet_save_position (info, info->gconf_key, TRUE);
+	}
+
+	g_slist_free (queued_position_saves);
+	queued_position_saves = NULL;
+
+	return FALSE;
+}
+
 void
 panel_applet_save_position (AppletInfo *applet_info,
-			    const char *gconf_key)
+			    const char *gconf_key,
+			    gboolean    immediate)
 {
-	GConfClient *client;
-	const char  *profile;
-	char        *temp_key;
+	GConfClient  *client;
+	const char   *profile;
+	const char   *temp_key;
+	gboolean      right_stick;
+	int           position;
+
+	g_return_if_fail (applet_info != NULL);
+
+	if (!immediate) {
+		if (!queued_position_source)
+			queued_position_source =
+				g_timeout_add (1000,
+					       (GSourceFunc) panel_applet_position_save_timeout,
+					       NULL);
+
+		if (!g_slist_find (queued_position_saves, applet_info))
+			queued_position_saves =
+				g_slist_prepend (queued_position_saves, applet_info);
+
+		return;
+	}
 
 	client  = panel_gconf_get_client ();
 	profile = panel_gconf_get_profile ();
 
+	right_stick = panel_applet_get_right_stick (applet_info);
+	temp_key = panel_applet_get_full_gconf_key (
+			applet_info->type, profile, gconf_key, "panel_right_stick");
+	gconf_client_set_bool (client, temp_key, right_stick, NULL);
+
+	position = panel_applet_get_position (applet_info);
+	if (right_stick)
+		position = PANEL_WIDGET (applet_info->widget->parent)->size - position;
+
 	temp_key = panel_applet_get_full_gconf_key (
 			applet_info->type, profile, gconf_key, "position");
-	gconf_client_set_int (client, temp_key, panel_applet_get_position (applet_info), NULL);
-	g_free (temp_key);
+	gconf_client_set_int (client, temp_key, position, NULL);
 
 	temp_key = panel_applet_get_full_gconf_key (
-			applet_info->type, profile, gconf_key, "panel-id");
+			applet_info->type, profile, gconf_key, "panel_id");
 	gconf_client_set_string (client, temp_key, panel_applet_get_panel_id (applet_info), NULL);
-	g_free (temp_key);
 
-	temp_key = panel_applet_get_full_gconf_key (
-			applet_info->type, profile, gconf_key, "panel-right-stick");
-	gconf_client_set_bool (client, temp_key, panel_applet_get_right_stick (applet_info), NULL);
-	g_free (temp_key);
 
 	gconf_client_suggest_sync (client, NULL);
 }
@@ -1022,16 +997,16 @@ panel_applet_save_to_gconf (AppletInfo *applet_info)
 {
 	GConfClient *client;
 	const char  *profile;
-	char        *temp_key;
+	const char  *temp_key;
 	GSList      *id_list, *l;
 
 	client  = panel_gconf_get_client ();
 	profile = panel_gconf_get_profile ();
 
 	if (applet_info->type == APPLET_BONOBO)
-		temp_key = panel_gconf_general_key (profile, "applet-id-list");
+		temp_key = panel_gconf_general_key (profile, "applet_id_list");
 	else
-		temp_key = panel_gconf_general_key (profile, "object-id-list");
+		temp_key = panel_gconf_general_key (profile, "object_id_list");
 
 	id_list = gconf_client_get_list (client, temp_key, GCONF_VALUE_STRING, NULL);
 
@@ -1039,7 +1014,7 @@ panel_applet_save_to_gconf (AppletInfo *applet_info)
 		applet_info->gconf_key = gconf_unique_key ();
 
 	for (l = id_list; l; l = l->next)
-		if (!strcmp (applet_info->gconf_key, (char *) l->data))
+		if (strcmp (applet_info->gconf_key, (char *) l->data) == 0)
 			break;
 
 	if (!l) {
@@ -1048,18 +1023,16 @@ panel_applet_save_to_gconf (AppletInfo *applet_info)
 		gconf_client_set_list (client, temp_key, GCONF_VALUE_STRING, id_list, NULL);
 	}
 
-	g_free (temp_key);
 	panel_g_slist_deep_free (id_list);
 
 	temp_key = panel_applet_get_full_gconf_key (applet_info->type, profile,
 						    applet_info->gconf_key,
-						    "object-type");
+						    "object_type");
 	gconf_client_set_string (client, temp_key,
 				 gconf_enum_to_string (object_type_enum_map, applet_info->type),
 				 NULL);
-	g_free (temp_key);
 
-	panel_applet_save_position (applet_info, applet_info->gconf_key);
+	panel_applet_save_position (applet_info, applet_info->gconf_key, TRUE);
 
 	switch (applet_info->type) {
 	case APPLET_BONOBO:
@@ -1070,10 +1043,6 @@ panel_applet_save_to_gconf (AppletInfo *applet_info)
 		drawer_save_to_gconf ((Drawer *) applet_info->data,
 				      applet_info->gconf_key);
 		break;
-	case APPLET_SWALLOW:
-		swallow_save_to_gconf ((Swallow *) applet_info->data,
-				       applet_info->gconf_key);
-		break;
 	case APPLET_MENU:
 		menu_save_to_gconf ((Menu *) applet_info->data,
 				    applet_info->gconf_key);
@@ -1084,7 +1053,6 @@ panel_applet_save_to_gconf (AppletInfo *applet_info)
 		break;
 	case APPLET_LOGOUT: /* no object specific stuff to save */
 	case APPLET_LOCK:
-	case APPLET_STATUS:
 	default:
 		break;
 	}
@@ -1135,6 +1103,8 @@ panel_applet_register (GtkWidget      *applet,
 		g_object_set_data (G_OBJECT (applet),
 				   PANEL_APPLET_ASSOC_PANEL_KEY, assoc_panel);
 		assoc_panel->master_widget = applet;
+		g_object_add_weak_pointer (
+			G_OBJECT (applet), (gpointer *) &assoc_panel->master_widget);
 	}
 
 	g_object_set_data (G_OBJECT (applet),
@@ -1165,21 +1135,18 @@ panel_applet_register (GtkWidget      *applet,
 	if (exactpos)
 		insert_at_pos = TRUE;
 
-
-	
-
-	if (panel_widget_add_full (panel, applet, newpos, TRUE,
-				   insert_at_pos, expand_major, expand_minor)==-1) {
+	if (panel_widget_add (panel, applet, newpos,
+			      insert_at_pos, expand_major, expand_minor) == -1) {
 		GSList *list;
 		for(list = panels; list != NULL; list = g_slist_next(list))
-			if (panel_widget_add_full (panel, applet, 0,
-						   TRUE, TRUE, expand_major, expand_minor)!=-1)
+			if (panel_widget_add (panel, applet, 0, TRUE,
+					      expand_major, expand_minor) != -1)
 				break;
 		if(!list) {
 			/*can't put it anywhere, clean up*/
 			info->widget = NULL;
 			gtk_widget_destroy (applet);
-			panel_applet_clean (info);
+			panel_applet_clean (info, TRUE);
 			g_warning(_("Can't find an empty spot"));
 			return NULL;
 		}
