@@ -45,6 +45,7 @@ GSList *panels = NULL; /*other panels we might want to move the applet to*/
 /*there  can universally be only one applet being dragged since we assume
 we only have one mouse :) */
 gboolean panel_applet_in_drag = FALSE;
+static GtkWidget *saved_focus_widget = NULL;
 
 /* Commie mode! */
 extern gboolean commie_mode;
@@ -72,11 +73,6 @@ static void panel_widget_style_set      (GtkWidget        *widget,
 					 GtkStyle         *previous_style);
 static void panel_widget_realize        (GtkWidget        *widget);
 
-static gboolean panel_widget_focus_in_event (GtkWidget        *widget,
-					     GdkEventFocus    *event);
-static gboolean panel_widget_focus_out_event(GtkWidget        *widget,
-					     GdkEventFocus    *event);
-
 static void panel_widget_push_move_applet   (PanelWidget      *panel,
                                              GtkDirectionType  dir);
 static void panel_widget_switch_move_applet (PanelWidget      *panel,
@@ -86,6 +82,7 @@ static void panel_widget_free_move_applet   (PanelWidget      *panel,
 static void panel_widget_tab_move           (PanelWidget      *panel,
                                              gboolean          next);
 static void panel_widget_end_move           (PanelWidget      *panel);
+static gboolean panel_widget_popup_panel_menu   (PanelWidget      *panel);
 static gboolean panel_widget_real_focus     (GtkWidget        *widget,
                                              GtkDirectionType  direction);
 
@@ -177,6 +174,7 @@ enum {
 	FREE_MOVE_SIGNAL,
 	TAB_MOVE_SIGNAL,
 	END_MOVE_SIGNAL,
+	POPUP_PANEL_MENU_SIGNAL,
 	LAST_SIGNAL
 };
 
@@ -193,9 +191,6 @@ add_tab_bindings (GtkBindingSet    *binding_set,
   	gtk_binding_entry_add_signal (binding_set, GDK_KP_Tab, modifiers,
 				      "tab_move", 1,
 				      G_TYPE_BOOLEAN, next);
-  	gtk_binding_entry_add_signal (binding_set, GDK_ISO_Left_Tab, modifiers,
-				      "tab_move", 1,
-				      G_TYPE_BOOLEAN, next);
 }
 
 static void
@@ -222,6 +217,7 @@ add_all_move_bindings (PanelWidget *panel)
 {
 	GtkWidgetClass *class;
 	GtkBindingSet *binding_set;
+	GtkWidget *focus_widget;
 
 	class = GTK_WIDGET_GET_CLASS (panel);
 
@@ -250,6 +246,44 @@ add_all_move_bindings (PanelWidget *panel)
 	gtk_binding_entry_add_signal (binding_set,
                                       GDK_space, 0,
                                       "end_move", 0);
+
+	focus_widget = gtk_window_get_focus (GTK_WINDOW (panel->panel_parent));
+	if (GTK_IS_SOCKET (focus_widget)) {
+		/*
+		 * If the focus widget is a GtkSocket, i.e. the
+		 * focus is in an applet in another process then
+		 * key bindings do not work. We get around this by
+		 * by setting the focus to the PanelWidget for the
+		 * duration of the move.
+		 */
+		GTK_WIDGET_SET_FLAGS (panel, GTK_CAN_FOCUS);
+		gtk_widget_grab_focus (GTK_WIDGET (panel));
+		saved_focus_widget = focus_widget;
+	}
+}
+
+static void
+panel_widget_force_grab_focus (GtkWidget *widget)
+{
+	gboolean can_focus = GTK_WIDGET_CAN_FOCUS (widget);
+	/*
+	 * This follows what gtk_socket_claim_focus() does
+	 */
+	if (!can_focus)
+		GTK_WIDGET_SET_FLAGS (widget, GTK_CAN_FOCUS);
+	gtk_widget_grab_focus (widget);
+	if (!can_focus)
+		GTK_WIDGET_UNSET_FLAGS (widget, GTK_CAN_FOCUS);
+}
+
+static void
+panel_widget_reset_saved_focus (PanelWidget *panel)
+{
+	if (saved_focus_widget) {
+		GTK_WIDGET_UNSET_FLAGS (panel, GTK_CAN_FOCUS);
+		panel_widget_force_grab_focus (saved_focus_widget);
+		saved_focus_widget = NULL;
+	}
 }
 
 static void
@@ -259,7 +293,6 @@ remove_tab_bindings (GtkBindingSet    *binding_set,
 {
 	gtk_binding_entry_clear (binding_set, GDK_Tab, modifiers);
   	gtk_binding_entry_clear (binding_set, GDK_KP_Tab, modifiers);
-  	gtk_binding_entry_clear (binding_set, GDK_ISO_Left_Tab, modifiers);
 }
 
 static void
@@ -282,6 +315,8 @@ remove_all_move_bindings (PanelWidget *panel)
 
 	binding_set = gtk_binding_set_by_class (class);
 
+	panel_widget_reset_saved_focus (panel);
+
 	remove_move_bindings (binding_set, GDK_SHIFT_MASK);
 	remove_move_bindings (binding_set, GDK_CONTROL_MASK);
 	remove_move_bindings (binding_set, GDK_MOD1_MASK);
@@ -303,6 +338,9 @@ panel_widget_class_init (PanelWidgetClass *class)
 	GObjectClass *gobject_class = (GObjectClass*) class;
 	GtkWidgetClass *widget_class = (GtkWidgetClass*) class;
 	GtkContainerClass *container_class = (GtkContainerClass*) class;
+	GtkBindingSet *binding_set;
+
+	binding_set = gtk_binding_set_by_class (class);
 
 	panel_widget_signals[ORIENT_CHANGE_SIGNAL] =
                 g_signal_new ("orient_change",
@@ -449,6 +487,17 @@ panel_widget_class_init (PanelWidgetClass *class)
                               G_TYPE_NONE,
                               0);
 
+	panel_widget_signals[POPUP_PANEL_MENU_SIGNAL] =
+		g_signal_new ("popup_panel_menu",
+                              G_TYPE_FROM_CLASS (class),
+                              G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+                              G_STRUCT_OFFSET (PanelWidgetClass, popup_panel_menu),
+                              NULL,
+                              NULL,
+                              panel_marshal_BOOLEAN__VOID,
+                              G_TYPE_BOOLEAN,
+                              0);
+
 	class->orient_change = NULL;
 	class->size_change = NULL;
 	class->applet_move = NULL;
@@ -461,6 +510,7 @@ panel_widget_class_init (PanelWidgetClass *class)
 	class->free_move = panel_widget_free_move_applet;
 	class->tab_move = panel_widget_tab_move;
 	class->end_move = panel_widget_end_move;
+	class->popup_panel_menu = panel_widget_popup_panel_menu;
 
 	object_class->destroy = panel_widget_destroy;
 	gobject_class->finalize = panel_widget_finalize;
@@ -470,12 +520,13 @@ panel_widget_class_init (PanelWidgetClass *class)
 	widget_class->realize = panel_widget_realize;
 	widget_class->expose_event = panel_widget_expose;
 	widget_class->style_set = panel_widget_style_set;
-	widget_class->focus_in_event = panel_widget_focus_in_event;
-	widget_class->focus_out_event = panel_widget_focus_out_event;
 	widget_class->focus = panel_widget_real_focus;
 
 	container_class->add = panel_widget_cadd;
 	container_class->remove = panel_widget_cremove;
+
+	gtk_binding_entry_add_signal (binding_set, GDK_F10, GDK_CONTROL_MASK,
+				      "popup_panel_menu", 0);
 }
 
 static void
@@ -548,6 +599,28 @@ run_up_forbidden(PanelWidget *panel,
 	(*runfunc)(panel,panel);
 }
 
+static void
+panel_widget_reset_focus (GtkContainer *container,
+                          GtkWidget    *widget)
+{
+	PanelWidget *panel = PANEL_WIDGET (container);
+
+	if (container->focus_child == widget) {
+		GtkWidget *panelw = panel->panel_parent;
+		gboolean return_val;
+
+		g_signal_emit_by_name (panelw, "focus",
+				       GTK_DIR_TAB_FORWARD,
+				       &return_val);
+		/*
+		 * The last object on the panel has been deleted so
+		 * put focus back on the panel.
+		 */
+		if (!gtk_window_get_focus (GTK_WINDOW (panelw))) {
+			panel_widget_focus (panel);	
+		}
+	}
+}
 
 static void
 panel_widget_cadd (GtkContainer *container,
@@ -594,6 +667,8 @@ panel_widget_cremove (GtkContainer *container, GtkWidget *widget)
 	if (p != NULL)
 		run_up_forbidden (p, remove_panel_from_forbidden);
 
+	panel_widget_reset_focus (container, widget);
+
 	if(panel->currently_dragged_applet == ad)
 		panel_widget_applet_drag_end(panel);
 
@@ -713,15 +788,15 @@ allocate_dirty_child(gpointer data)
 		}
 		ad->cells = chreq.width; 
 		challoc.x = ad->pos;
-		challoc.y = (GTK_WIDGET(panel)->allocation.height -
-			     challoc.height) / 2;
+		challoc.y = GTK_WIDGET(panel)->allocation.height / 2 -
+			     challoc.height / 2;
 	} else {
 		if (ad->expand_minor) {
 			challoc.width = GTK_WIDGET (panel)->allocation.width;
 		}
 		ad->cells = chreq.height;
-		challoc.x = (GTK_WIDGET(panel)->allocation.width -
-			     challoc.width) / 2;
+		challoc.x = GTK_WIDGET(panel)->allocation.width / 2 -
+			    challoc.width / 2;
 		challoc.y = ad->pos;
 	}
 
@@ -1309,13 +1384,13 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 				}
 				ad->cells = chreq.width;
 				challoc.x = ad->pos;
-				challoc.y = (allocation->height - challoc.height) / 2;
+				challoc.y = allocation->height / 2 - challoc.height / 2;
 			} else {
 				if (ad->expand_minor) {
 					challoc.width = allocation->width;
 				}
 				ad->cells = chreq.height;
-				challoc.x = (allocation->width - challoc.width) / 2;
+				challoc.x = allocation->width / 2 - challoc.width / 2;
 				challoc.y = ad->pos;
 			}
 			ad->min_cells  = ad->cells;
@@ -1389,13 +1464,13 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 					challoc.height = allocation->height;
 				}
 				challoc.x = ad->pos;
-				challoc.y = (allocation->height - challoc.height) / 2;
+				challoc.y = allocation->height / 2 - challoc.height / 2;
 			} else {
 				challoc.height = ad->cells;
 				if (ad->expand_minor) {
 					challoc.width = allocation->width;
 				}
-				challoc.x = (allocation->width - challoc.width) / 2;
+				challoc.x = allocation->width / 2 - challoc.width / 2;
 				challoc.y = ad->pos;
 			}
 			ad->dirty = FALSE;
@@ -1698,9 +1773,6 @@ panel_widget_finalize (GObject *obj)
 	g_free (panel->unique_id);
 	panel->unique_id = NULL;
 
-	if (panel->key_event)
-		gdk_event_free ((GdkEvent *)panel->key_event);
-
 	if (G_OBJECT_CLASS (panel_widget_parent_class)->finalize)
 		G_OBJECT_CLASS (panel_widget_parent_class)->finalize (obj);
 }
@@ -2679,6 +2751,7 @@ panel_widget_reparent (PanelWidget *old_panel,
 		       int pos)
 {
 	AppletData *ad;
+	GtkWidget *focus_widget = NULL;
 
 	g_return_val_if_fail(PANEL_IS_WIDGET(old_panel),-1);
 	g_return_val_if_fail(PANEL_IS_WIDGET(new_panel),-1);
@@ -2695,8 +2768,25 @@ panel_widget_reparent (PanelWidget *old_panel,
 
 	ad->no_die++;
 
+	panel_widget_reset_saved_focus (old_panel);
+	if (GTK_CONTAINER (old_panel)->focus_child == applet) {
+		focus_widget = gtk_window_get_focus (GTK_WINDOW (old_panel->panel_parent));
+	}
 	gtk_widget_reparent (applet, GTK_WIDGET (new_panel));
 
+	if (GTK_WIDGET_CAN_FOCUS (new_panel))
+		GTK_WIDGET_UNSET_FLAGS (new_panel, GTK_CAN_FOCUS);
+	if (focus_widget) {
+		panel_widget_force_grab_focus (focus_widget);
+	} else {
+		gboolean return_val;
+
+		g_signal_emit_by_name (applet, "focus",
+				       GTK_DIR_TAB_FORWARD,
+				       &return_val);
+	}
+ 	gtk_window_present (GTK_WINDOW (new_panel->panel_parent));
+
 	gdk_flush();
 
 	ad->no_die--;
@@ -2884,6 +2974,8 @@ panel_widget_tab_move (PanelWidget *panel,
 	AppletData       *ad;
 	GSList *li;
 	PanelWidget *new_panel = NULL;
+	PanelWidget *previous_panel = NULL;
+	PanelWidget *first_panel = panels->data;
 
 	ad = panel->currently_dragged_applet;
 
@@ -2891,15 +2983,20 @@ panel_widget_tab_move (PanelWidget *panel,
 		return;	
 	
 	for (li = panels; li; li = li->next) {
-		PanelWidget *previous_panel = NULL;
 		PanelWidget *panel_in_list = li->data;
 
 		if (panel_in_list == panel) {
 			if (next) {
 				if (li->next)
 					new_panel = li->next->data;
+				else
+					new_panel = ((GSList *)panels)->data;
+				
 			} else {
-				new_panel = previous_panel;
+				if (previous_panel)
+					new_panel = previous_panel;
+				else
+					continue;
 			}
 			break;
 		} else {		
@@ -2908,8 +3005,13 @@ panel_widget_tab_move (PanelWidget *panel,
 		}
 	}
 	g_return_if_fail (li);
-        if (new_panel)
+	if (!new_panel) {
+		if (previous_panel)
+			new_panel = previous_panel;
+	}
+	if (new_panel && (new_panel != panel)) {
 		panel_widget_reparent (panel, new_panel, ad->applet, 0);
+	}
 }
 
 static void
@@ -2918,59 +3020,21 @@ panel_widget_end_move (PanelWidget *panel)
 	panel_widget_applet_drag_end (panel);
 }
 
-static gboolean
-panel_widget_focus_in_event (GtkWidget *widget, GdkEventFocus *event)
-{
-	/*
-	 * FIXME: we need to figure out something sensible
-	 *        to do to indicate focus
-	 */
-
-	GTK_WIDGET_CLASS (panel_widget_parent_class)->focus_in_event (widget, event);
-
-	return FALSE;
-}
-
-static gboolean
-panel_widget_focus_out_event (GtkWidget *widget, GdkEventFocus *event)
-{
-	PanelWidget *panel_widget = PANEL_WIDGET (widget);
-
-	if (GTK_WINDOW (panel_widget->panel_parent)->has_focus)
-		GTK_WIDGET_UNSET_FLAGS (widget, GTK_CAN_FOCUS);
-
-	GTK_WIDGET_CLASS (panel_widget_parent_class)->focus_out_event (widget, event);
-
-	return FALSE;
-}
-
 static gboolean
 panel_widget_real_focus (GtkWidget        *widget,
                          GtkDirectionType  direction)
 {
-	PanelWidget *panel = PANEL_WIDGET (widget);
-
-	if (panel->key_event) {
-		if (panel->key_event->state & GDK_CONTROL_MASK) {
-			/*
-	 		 * Ctrl+Tab was pressed when focus was in applet in
-			 * another process
-			 */
-			panel_widget_focus (PANEL_WIDGET (widget));
-			return TRUE;
-		}
-	}
-
-	if (GTK_WIDGET_HAS_FOCUS (widget)) {
+	if (GTK_WIDGET_HAS_FOCUS (widget) && GTK_FIXED (widget)->children) {
 		GTK_WIDGET_UNSET_FLAGS (widget, GTK_CAN_FOCUS);
 	}
 	return GTK_WIDGET_CLASS (panel_widget_parent_class)->focus (widget, direction);
 }
 
-void panel_widget_focus (PanelWidget *panel)
+void 
+panel_widget_focus (PanelWidget *panel)
 {
 	/*
-	 * Set the focus back on the panel; we unset the focus child so that
+         * Set the focus back on the panel; we unset the focus child so that
 	 * the next time focus is inside the panel we do not remember the
 	 * previously focused child. We also need to set GTK_CAN_FOCUS flag
 	 * on the panel as it is unset when this function is called.
@@ -2978,10 +3042,11 @@ void panel_widget_focus (PanelWidget *panel)
 	if (!DRAWER_IS_WIDGET (panel->panel_parent)) {
 		gtk_container_set_focus_child (GTK_CONTAINER (panel), NULL);
 		GTK_WIDGET_SET_FLAGS (panel, GTK_CAN_FOCUS);
-      		gtk_widget_grab_focus (GTK_WIDGET (panel));
+		gtk_widget_grab_focus (GTK_WIDGET (panel));
 	}
 }
 
+
 PanelOrient
 panel_widget_get_applet_orient (PanelWidget *panel)
 {
@@ -2994,13 +3059,11 @@ panel_widget_get_applet_orient (PanelWidget *panel)
 	return basep_widget_get_applet_orient (BASEP_WIDGET (panel->panel_parent));
 }
 
-void 
-panel_widget_save_key_event (PanelWidget *panel,
-			     GdkEventKey *key_event)
+static gboolean 
+panel_widget_popup_panel_menu (PanelWidget *panel)
 {
-	if (panel->key_event) {
-		gdk_event_free ((GdkEvent *)panel->key_event);
-	}
-	panel->key_event = (GdkEventKey *) gdk_event_copy ((GdkEvent *)key_event);
-}
+	gboolean ret_val;
 
+	g_signal_emit_by_name (panel, "popup_menu", &ret_val);
+	return ret_val;
+}
