@@ -54,6 +54,11 @@ struct _AppletData
 	int		no_die; /* if >0 never send the about to die
 				   signal, an int and not a bool for
 				   nesting reasons */
+
+	/* Valid size ranges for expanded applets */
+	int *           size_hints; 
+	int             size_hints_len;
+  
 };
 
 struct _PanelWidget
@@ -133,6 +138,7 @@ struct _PanelWidgetClass
 	void (* tab_move) (PanelWidget	*panel,
                            gboolean	 next);
 	void (* end_move) (PanelWidget	*panel);
+	gboolean (* popup_panel_menu) (PanelWidget	*panel);
 
 };
 
