@@ -33,6 +33,7 @@
 #include <gdk/gdkx.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkbindings.h>
+#include <gtk/gtktooltips.h>
 #include <bonobo/bonobo-ui-util.h>
 #include <bonobo/bonobo-main.h>
 #include <bonobo/bonobo-types.h>
@@ -480,6 +481,71 @@ panel_applet_popup_menu (PanelApplet *applet)
 	return _panel_applet_popup_menu (applet, 3, GDK_CURRENT_TIME);
 }
 
+static void
+panel_applet_size_request (GtkWidget *widget, GtkRequisition *requisition)
+{
+	PanelApplet *applet = PANEL_APPLET (widget);
+	GtkBin *bin = GTK_BIN (widget);
+	gint focus_width, focus_pad;
+
+	requisition->width = GTK_CONTAINER (widget)->border_width * 2;
+	requisition->height = GTK_CONTAINER (widget)->border_width * 2;
+
+	gtk_widget_style_get (widget,
+			      "focus-line-width", &focus_width,
+			      "focus-padding", &focus_pad,
+			      NULL);
+
+	if (bin->child && GTK_WIDGET_VISIBLE (bin->child)) {
+		GtkRequisition child_requisition;
+
+		gtk_widget_size_request (bin->child, &child_requisition);
+
+		requisition->width += child_requisition.width;
+		requisition->height += child_requisition.height;
+	}
+	requisition->width += 2 * (focus_pad + 2 * focus_width);
+	requisition->height += 2 * (focus_pad + 2 * focus_width);
+}
+
+static void
+panel_applet_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
+{
+	PanelApplet *applet = PANEL_APPLET (widget);
+	gint focus_width, focus_pad;
+	gint x, y, width, height;
+	gint border_width;
+	GtkBin *bin;
+	GtkAllocation child_allocation;
+
+	gtk_widget_style_get (widget,
+			      "focus-line-width", &focus_width,
+			      "focus-padding", &focus_pad,
+			      NULL);
+	border_width = GTK_CONTAINER (widget)->border_width;
+
+	widget->allocation = *allocation;
+	bin = GTK_BIN (widget);
+
+ 	child_allocation.x = focus_width + 2 * focus_pad;
+ 	child_allocation.y = focus_width + 2 * focus_pad;
+	child_allocation.width = MAX (allocation->width - border_width * 2, 0);
+	child_allocation.height = MAX (allocation->height - border_width * 2, 0);
+
+	if (GTK_WIDGET_REALIZED (widget))
+		gdk_window_move_resize (widget->window,
+					allocation->x + GTK_CONTAINER (widget)->border_width,
+					allocation->y + GTK_CONTAINER (widget)->border_width,
+					child_allocation.width,
+					child_allocation.height);
+	child_allocation.width = MAX (child_allocation.width - 2 * (focus_width + 2 * focus_pad), 0);
+	child_allocation.height = MAX (child_allocation.height - 2 * (focus_width + 2 * focus_pad), 0);
+
+	if (bin->child)
+		gtk_widget_size_allocate (bin->child, &child_allocation);
+}
+
+
 static gboolean
 panel_applet_expose (GtkWidget      *widget,
 		     GdkEventExpose *event) 
@@ -489,15 +555,26 @@ panel_applet_expose (GtkWidget      *widget,
 
 	GTK_WIDGET_CLASS (parent_class)->expose_event (widget, event);
 
-        if (GTK_WIDGET_HAS_FOCUS (widget))
+        if (GTK_WIDGET_HAS_FOCUS (widget)) {
+		gint focus_width, focus_pad;
+		gint x, y, width, height;
+		gint border_width;
+
+		gtk_widget_style_get (widget,
+				      "focus-line-width", &focus_width,
+				      "focus-padding", &focus_pad,
+				      NULL);
+		border_width = GTK_CONTAINER (widget)->border_width;
+		x = widget->allocation.x + focus_pad;
+		y = widget->allocation.y + focus_pad;
+		width = widget->allocation.width - 2 * border_width - (focus_width + 2 * focus_pad);
+		height = widget->allocation.height - 2 * border_width - (focus_width +  2 * focus_pad);
+
 		gtk_paint_focus (widget->style, widget->window,
                                  GTK_WIDGET_STATE (widget),
                                  &event->area, widget, "panel_applet",
-                                 widget->allocation.x + 1,
-                                 widget->allocation.y + 1,
-                                 widget->allocation.width - 3,
-                                 widget->allocation.height - 3);
-
+                                 x, y, width, height);
+	}
 	return FALSE;
 }                
 
@@ -516,12 +593,24 @@ panel_applet_focus (GtkWidget        *widget,
 		return FALSE;
 
 	previous_focus_child = GTK_CONTAINER (widget)->focus_child;
+	 if (!previous_focus_child && !GTK_WIDGET_HAS_FOCUS (widget)) {
+		GtkTooltipsData *tooltip;
+
+		tooltip = gtk_tooltips_data_get (widget);
+		if (tooltip) {
+			GTK_WIDGET_SET_FLAGS (widget, GTK_CAN_FOCUS);
+			gtk_widget_grab_focus (widget);
+			GTK_WIDGET_UNSET_FLAGS (widget, GTK_CAN_FOCUS);
+			return TRUE;
+		}
+	}
 	ret = GTK_WIDGET_CLASS (parent_class)->focus (widget, dir);
 	if (!ret && !previous_focus_child) {
- 		if (!GTK_WIDGET_CAN_FOCUS (widget))  {
+ 		if (!GTK_WIDGET_HAS_FOCUS (widget))  {
 			/*
 			 * Applet does not have a widget which can focus so set
-			 * the focus on the applet.
+			 * the focus on the applet unless it aleready had focus
+			 * because it had a tooltip.
 			 */ 
 			GTK_WIDGET_SET_FLAGS (widget, GTK_CAN_FOCUS);
 			gtk_widget_grab_focus (widget);
@@ -1057,6 +1146,8 @@ panel_applet_class_init (PanelAppletClass *klass,
 	klass->move_focus_out_of_applet = panel_applet_move_focus_out_of_applet;
 
 	widget_class->button_press_event = panel_applet_button_press;
+	widget_class->size_request = panel_applet_size_request;
+	widget_class->size_allocate = panel_applet_size_allocate;
 	widget_class->expose_event = panel_applet_expose;
 	widget_class->focus = panel_applet_focus;
 
